<?php
/**
 * Save selection from the client as post meta data
 * This function will be called from the front end (client view)
 *
 * @since 0.4.0
 */
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly


function picu_send_selection() {

	// Nonce check!
	if ( ! check_ajax_referer( 'picu-ajax-security', 'security', false ) ) {
		picu_send_json( 'error', __( '<strong>Error:</strong> Nonce check failed.<br />Refresh your browser window.', 'picu' ) );
	}

	// Sanitize and validate post id
	$postid = sanitize_key( $_POST['postid'] );
	// Does this collection exist?
	if ( ! is_string( get_post_status( $postid ) ) ) {
		picu_send_json( 'error', __( 'Error: Post id is not set.', 'picu' ) );
	}

	// Check post status / collection is already approved
	$poststatus = get_post_status( $postid );

	// Post status needs to be 'publish' or 'sent'. In all other cases, a selection may not be saved
	if ( 'approved' == $poststatus ) {
		picu_send_json( 'error', __( 'Error: Collection is already approved.', 'picu' ) );
	}

	// Allow for some additional validation
	do_action( 'picu_send_selection_valudation', $_POST );

	// Sanitize selection
	if ( isset( $_POST['selection'] ) AND ! empty( $_POST['selection'] ) ) {
		$temp = $_POST['selection'];
		$selection = array();
		foreach ( $temp as $id ) {
			// Ids must be integer values, handing them over as strings
			$selection[] = strval( intval( $id ) );
		}
	}
	else {
		$selection = false;
	}

	// Sanitize markers
	if ( isset( $_POST['markers'] ) AND ! empty( $_POST['markers'] ) ) {

		$markers = $_POST['markers'];

		function picu_ajax_sanitize_comment( &$item, $key ) {
			if ( $key == 'comment' ) {
				$item = sanitize_text_field( $item );
			}
		}

		array_walk_recursive( $markers, 'picu_ajax_sanitize_comment' );
	}
	else {
		$markers = '';
	}

	// Handle ident parameter
	// TODO: Also account for cookie, once we implemented that
	if ( isset( $_POST['ident'] ) AND $_POST['ident'] != 'false' ) {

		$ident = sanitize_key( $_POST['ident'] );
		$collection_hashes = get_post_meta( $postid, '_picu_collection_hashes', true );

		// Check if email is part of the hashes
		if ( is_array( $collection_hashes ) AND ! array_key_exists( $ident, $collection_hashes ) ) {
			$ident = false;
		}
	}
	else {
		$ident = false;
	}

	// Prepare array, that is saved as post meta
	$save = array(
		'selection' => $selection,
		'time' => time(),
		'markers' => $markers
	);

	// Save approval message
	$approval_message = '';
	if ( ! empty( $_POST['approval_fields']['picu_approval_message']['value'] ) ) {
		$approval_message = trim( implode( "\n", array_map( 'sanitize_text_field', explode( "\n", stripslashes( $_POST['approval_fields']['picu_approval_message']['value'] ) ) ) ) );
		$save['approval_fields']['picu_approval_message']['value'] = $approval_message;
		$save['approval_fields']['picu_approval_message']['label'] = __( 'Message', 'picu' );
	}

	// Filter the $save array
	$save = apply_filters( 'picu_save_selection', $save, $_POST );

	// Save selection, send result back to the user
	if ( $ident ) {
		$response = update_post_meta( $postid, '_picu_collection_selection_' . $ident, $save );
	}
	else {
		$response = update_post_meta( $postid, '_picu_collection_selection', $save );
	}

	// It worked
	if ( $response >= 1 ) {

		// Switch collection status to "approved"
		if ( is_array( $selection ) AND count( $selection ) > 0 AND isset( $_POST['intent'] ) AND $_POST['intent'] == 'approve' ) {
			if ( $ident ) {
				$collection_hashes[$ident]['status'] = 'approved';
				$collection_hashes[$ident]['time'] = time();
				
				update_post_meta( $postid, '_picu_collection_hashes', $collection_hashes );

				// Check if all other users are already approved
				$are_we_done = true;
				foreach( $collection_hashes as $hash ) {
					if ( $hash['status'] != 'approved' ) {
						$are_we_done = false;
					}
				}

				// If everyone is done, set post status to approved, inform photographer
				if ( $are_we_done ) {
					picu_update_post_status( $postid, 'approved' );
					picu_send_approval_mail( $postid );

					do_action( 'picu_after_approval', $postid );	
				}

				picu_send_json( 'success', __( 'You successfully approved this collection.', 'picu' ) );
			}
			else {
				// Set the post status to "approved"
				picu_update_post_status( $postid, 'approved' );

				// Inform the photographer about the approval
				picu_send_approval_mail( $postid, $approval_message );

				do_action( 'picu_after_approval', $postid );

				picu_send_json( 'success', __( 'You successfully approved this collection.', 'picu' ) );
			}

		}
		// Return error if intent was to approve but no images was selected
		elseif ( is_array( $selection ) AND count( $selection ) <= 0 AND isset( $_POST['intent'] ) AND $_POST['intent'] == 'approve' ) {

			picu_send_json( 'error', __( 'Please select at least one image.', 'picu' ) );

		}
		// Regular "saved was a success" message
		else {
			picu_send_json( 'success', __( 'Your selection was saved.', 'picu' ) );
		}

	}
	// It didn't work
	else {
		// TODO: How about a more meaningful error message?
		picu_send_json( 'error', __( 'Error. Your selection could not be saved.', 'picu' ) );
	}

	exit;
}

add_action( 'wp_ajax_picu_send_selection', 'picu_send_selection' );
add_action( 'wp_ajax_nopriv_picu_send_selection', 'picu_send_selection' );




/**
 * Save visibility state of the picu pro meta box
 *
 * @since 1.3.1
 */
function picu_save_pro_box_state() {
	if ( ! check_ajax_referer( 'picu_ajax', 'security', false ) ) {
		picu_send_json( 'error', __( '<strong>Error:</strong> Nonce check failed.', 'picu' ) );
	}

	if ( isset( $_REQUEST['picu_hide_pro_box'] ) AND $_REQUEST['picu_hide_pro_box'] ==
'true' ) {
		set_transient( 'picu_pro_box_hidden_' . get_current_user_id(), true, YEAR_IN_SECONDS );
	}
	else {
		delete_transient( 'picu_pro_box_hidden_' . get_current_user_id() );
	}

	wp_send_json_success( $_REQUEST );

	exit;
}

add_action( 'wp_ajax_picu_save_pro_box_state', 'picu_save_pro_box_state' );


/**
 * Sends a JSON response
 * 
 * @param string $type Whether to send a 'success' or an 'error'
 * @param string $message (optional) Message to be sent back with the JSON data, default: 'An error occured'
 * @param string $button_text (optional) Text for the button we use in the error message, default: 'OK'
 * 
 * @since 1.6.5
 */
function picu_send_json( $type, $message = null, $button_text = null ) {

	if ( ! isset ( $message ) ) {
		if ( $type == 'success' ) {
			$message = __( 'Success', 'picu' );
		}
		else {
			$message = __( 'An error occured', 'picu' );
		}
	}

	if ( ! isset ( $button_text ) ) {
		$button_text = __( 'OK', 'picu' );
	}

	$return = array(
		'message' => $message,
		'button_text' => $button_text
	);

	if ( $type == 'success' ) {
		wp_send_json_success( $return );
	}
	elseif ( $type == 'error' ) {
		wp_send_json_error( $return );
	}
	exit;

}
