<?php
/**
 * Custom Post Type "Collections"
 *
 * @since 0.1.0
 */
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly


/**
 * Register Custom Post Type 'picu_collection'
 *
 * @since 0.1.0
 */
function picu_register_cpt_collection() {

	$labels = array(
		'name' => __( 'Collections', 'picu' ),
		'singular_name' => __( 'Collection', 'picu' ),
		'add_new' => __( 'New Collection', 'picu' ),
		'add_new_item' => __( 'New Collection', 'picu' ),
		'edit_item' => __( 'Edit Collection', 'picu' ),
		'new_item' => __( 'New Collection', 'picu' ),
		'view_item' => __( 'View Collection', 'picu' ),
		'search_items' => __( 'Search Collections', 'picu' ),
		'not_found' => __( 'No Collection Found', 'picu' ),
		'not_found_in_trash' => __( 'No Collection Found in Trash', 'picu' ),
		'parent_item_colon' => __( 'Parent Collection', 'picu' ),
		'menu_name' => __( 'All Collections', 'picu' ),
		'filter_items_list' => __( 'Filter collections list', 'picu' ),
		'items_list_navigation' => __( 'Collections list navigation', 'picu' ),
		'items_list' => __( 'Collections list', 'picu' )
	);

	/* translators: Base slug (part of the URL) for our custom post type. Cannot contain any special characters! */
	$picu_collection_slug = apply_filters( 'picu_collection_slug', _x( 'collections', 'picu collections slug', 'picu' ) );

	/* Make sure the slug is sanitzed */
	$picu_collection_slug = sanitize_title( $picu_collection_slug );

	$args = array(
		'labels' => $labels,
		'hierarchical' => false,
		'supports' => array( 'title' ),
		'public' => true,
		'show_ui' => true,
		'show_in_nav_menus' => false,
		'show_in_menu'  => 'picu',
		'menu_position' => 1,
		'publicly_queryable' => true,
		'exclude_from_search' => true,
		'has_archive' => false,
		'query_var' => true,
		'can_export' => true,
		'rewrite' => array(
			'slug' => $picu_collection_slug,
			'with_front' => false
		),
		'capabilities' => array(
			'edit_post'          => picu_capability(),
			'read_post'          => picu_capability(),
			'delete_post'        => picu_capability(),
			'edit_posts'         => picu_capability(),
			'edit_others_posts'  => picu_capability(),
			'delete_posts'       => picu_capability(),
			'publish_posts'      => picu_capability(),
			'read_private_posts' => picu_capability()
),
	);

	$args = apply_filters( 'picu_cpt_collection_args', $args );

	register_post_type( 'picu_collection', $args );

}

add_action( 'init', 'picu_register_cpt_collection' );


/**
 * Register custom post statuses
 *
 * @since 0.1.0
 */
function picu_collection_post_status() {

	register_post_status( 'sent', array(
		'label' => _x( 'Sent', 'post status name', 'picu' ),
		'public' => true,
		'exclude_from_search' => false,
		'show_in_admin_all_list' => true,
		'show_in_admin_status_list' => true,
		'label_count' => _n_noop( 'Sent <span class="count">(%s)</span>', 'Sent <span class="count">(%s)</span>', 'picu' ),
	) );

	register_post_status( 'approved', array(
		'label' => _x( 'Approved', 'post status name', 'picu' ),
		'public' => true,
		'exclude_from_search' => false,
		'show_in_admin_all_list' => true,
		'show_in_admin_status_list' => true,
		'label_count' => _n_noop( 'Approved <span class="count">(%s)</span>', 'Approved <span class="count">(%s)</span>', 'picu' ),
	) );
}

add_action( 'init', 'picu_collection_post_status' );


/**
 * Add a unique post slug to new collections
 *
 * @since 0.1.0
 */
function picu_add_unique_post_slug( $data, $postarr ) {

	// Check options, do we generate random slugs or not?
	$options = get_option( 'picu_settings' );
	$picu_collection_do_random_slug = true;

	if ( isset( $options['random_slugs'] ) AND $options['random_slugs'] == 'off' ) {
		$picu_collection_do_random_slug = false;
	}

	// Posibility to override with filter
	$picu_collection_do_random_slug = apply_filters( 'picu_collection_do_random_slug', $picu_collection_do_random_slug );

	// We only want our hashed slugs for post-type "collection"
	if ( $data['post_type'] == 'picu_collection' AND empty( $data['post_name'] ) AND $picu_collection_do_random_slug === true ) {
		$data['post_name'] = substr( md5( rand() ), 0, 5 );
	}

	return $data;
}

add_filter( 'wp_insert_post_data', 'picu_add_unique_post_slug', 99, 2 );


/**
 * Get our custom link to a collection, even before it is saved for the first time
 *
 * @return a string with the full url to a collection
 */
function get_draft_permalink( $post_id ) {

	// Load sample permalink and post-name into separate variables
	list( $permalink, $postname ) = get_sample_permalink( $post_id );

	// Check if post-name has not been set yet
	if ( empty( $postname ) ) {
		// Generate the post-name that will be used later on
		$postname = substr( md5( $post_id ), 0, 5 );
	}

	// Replace the placeholder in our URL
	return str_replace( '%pagename%', $postname, $permalink );

}


/**
 * Add the collection status to the collection list view
 *
 * @since 0.3.2
 */
function picu_collection_add_status_admin_column( $columns ) {

	// In order to set date as the last column we need to unset it first
	unset( $columns['date'] );

	// Add our custom columns to the default $columns
	$columns['picu_actions'] = _x( '<span class="screen-reader-text">Actions</span>', 'column header', 'picu' );
	$columns['approval_status'] = _x( 'Status', 'column header', 'picu' );
	$columns['date'] = __( 'Date' );

	// Return all columns including our custom column
	return $columns;
}

add_filter( 'manage_picu_collection_posts_columns', 'picu_collection_add_status_admin_column', 10 );


/**
 * Add content to our admin column
 *
 * @since 0.3.2
 */
function picu_column_collection_status( $column, $post_id ) {

	$post_status = get_post_status( $post_id );
	$img_filenames = picu_get_approved_filenames( $post_id );

	// Check that we are in the right column for approval status
	if ( $column == 'approval_status' ) {

		// Status URL, which will be linked to from the column
		$status_url = add_query_arg( 'post_status', $post_status );

		// Define the status messages for our column
		// depending on the status of each collection
		switch ( $post_status ) {
			case 'delivered':
				$title_description = __( 'This collection has been delivered to the client.', 'picu' );
				$status_class = 'picu-admin-status-delivered';
				break;
			case 'delivery-draft':
				$title_description = __( 'This collection is a delivery draft.', 'picu' );
				$status_class = 'picu-admin-status-delivery-draft';
				break;
			case 'approved':
				$title_description = __( 'This collection has been approved.', 'picu' );
				$status_class = 'picu-admin-status-approved';
				break;
			case 'sent':
				$title_description = __( 'This collection has been sent to the client for approval.', 'picu' );
				$status_class = 'picu-admin-status-sent';
				break;
			case 'publish':
				$title_description = __( 'This collection has been published but not yet sent to the client.', 'picu' );
				$status_class = 'picu-admin-status-publish';
				break;
			case 'draft':
				$title_description = __( 'This collection is a draft, which means it cannot be publicly accessed', 'picu' );
				$status_class = 'picu-admin-status-draft';
				break;
			default:
				$title_description = __( 'This collection is in the trash. You can either restore or permanently delete it.', 'picu' );
				$status_class = 'picu-admin-status-publish';
		}

		// Construct the final output
		echo '<a href="'.$status_url.'" class="picu-admin-status '.$status_class.'" title="'.$title_description.'"></a>';

	}

	if ( $column == 'picu_actions' ) {

		if ( ( $post_status == 'approved' OR $post_status == 'delivery-draft' OR $post_status == 'delivered' ) AND ! empty( $img_filenames ) ) {
		?>
			<a class="button button-primary" role="button" tabindex="0" href="<?php if ( $post_status == 'approved' OR $post_status == 'delivery-draft' OR $post_status == 'delivered' ) { echo admin_url( 'post.php?post=' . $post_id . '&action=edit&picu-download=picu-proof-file' ); } ?>"><?php _e( 'Download Proof', 'picu' ); ?> (.txt)</a>
		<?php }

	}
}

add_action( 'manage_picu_collection_posts_custom_column' , 'picu_column_collection_status', 10, 2 );


/**
 * Remove "Proteced" prefix from password protected collection title
 *
 * @since 0.9.0
 */
function picu_remove_protected_prefix( $title_text ) {
	global $post;
	if ( 'picu_collection' == $post->post_type ) {
		return '%s';
	}
	else {
		return $title_text;
	}
}

add_filter( 'protected_title_format', 'picu_remove_protected_prefix' );


/**
 * Remove picu collections from sitemaps
 * 
 * @since 1.5.0
 */
add_filter( 'wp_sitemaps_post_types', function( $post_types ) {
	unset( $post_types['picu_collection'] );
	return $post_types;
});
