<?php
/**
 * Picu Helper functions
 *
 * @since 0.5.0
 */
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly


/**
 * Add additional body classes for admin screens
 *
 * @since 0.3.2
 */
function picu_admin_body_class( $admin_body_class ) {

	// Get current admin screen
	$current_screen = get_current_screen();

	// Check if we are on a 'post.php' page
	if ( $current_screen->base == 'post' ) {
		// Add new class to the array;
		$admin_body_class .= ' post-status-' . get_post_status();
	}

	// return the array
	return $admin_body_class;

}

add_filter( 'admin_body_class', 'picu_admin_body_class' );


/**
 * Change the post_status
 *
 * @since 0.3.0
 */
function picu_update_post_status( $post_id, $status ) {

	if ( $status != 'delivered' && $status != 'delivery-draft' && $status != 'sent' && $status != 'approved' && $status != 'draft' ) {
		return $post_id;
	}

	$post_contents = array(
		'ID' => $post_id,
		'post_status' => $status
	);

	// Remove our mail and save function so we don't get trapped in a loop
	remove_action( 'save_post_picu_collection', 'picu_collection_send_mail' );

	// Update the post, which calls save_post again
	wp_update_post( $post_contents );

	// Re-add the function for mail and save after save_post has fired
	add_action( 'save_post_picu_collection', 'picu_collection_send_mail' );

}


/**
 * Send email to client and update collection history
 * This function will be used from the backend (photographer's view)
 *
 * @since 0.1.0
 */
function picu_collection_send_mail( $post_id, $post ) {

	// Only go ahead if our button was clicked
	if ( ! isset( $_POST['picu_sendmail'] ) )
		return $post_id;

	// Check if nonce is set
	if ( ! isset( $_POST['picu_collection_metabox_nonce'] ) )
		return $post_id;

	// Verify nonce
	if ( ! wp_verify_nonce( $_POST['picu_collection_metabox_nonce'], 'picu_collection_metabox' ) )
		return $post_id;

	// Abort if no title is set
	if ( ! $post->post_title )
		return $post_id;

	// Abort if there are no proof images, but the intent is proofing
	if ( isset( $_POST['picu_gallery_ids'] ) AND empty( $_POST['picu_gallery_ids'] ) ) {
		return $post_id;
	}

	// Abort if there are no delivery images, but the intent is delivery – and the delivery option is upload
	if ( isset( $_POST['delivery_image_ids'] ) AND empty( $_POST['delivery_image_ids'] ) AND $_POST['picu_delivery_option'] == 'upload' ) {
		return $post_id;
	}

	// Abort if autosave
	if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE )
		return $post_id;

	// Abort if the user doesn't have permissions
	if ( ! current_user_can( 'edit_post', $post_id ) )
		return $post_id;

	// Abort sending if there are error notifications
	$notifications = get_option( '_' . get_current_user_id() . '_picu_notifications' );

	if ( is_array( $notifications ) ) {
		foreach( $notifications as $notification ) {
			if ( strpos( $notification['type'], 'error' ) )  {
				return $post_id;
			}
		}
	}

	// Only send mail, if this method is selected
	if ( 'picu-send-email' == $_POST['picu_collection_share_method'] ) {

		// Abort if no email address is set
		if ( empty( $_POST['picu_collection_email_address'] ) AND empty( $_POST['picu_delivery_email_address'] ) )
			return $post_id;

		// Abort if no description is set
		if ( empty( $_POST['picu_collection_description'] ) AND empty( $_POST['picu_delivery_description'] ) )
			return $post_id;

		// Get recipient
		if ( ! empty( $_POST['picu_delivery_email_address'] ) ) {
			$email_address = $_POST['picu_delivery_email_address'];
		}
		else {
			$email_address = $_POST['picu_collection_email_address'];
		}

		// Check if there are multiple mail addresses
		if ( strpos( $email_address, ', ' ) ) {
			// Sanitize them individually
			$email_addresses = explode( ', ', $email_address );
			$email['recipient'] = array_filter( $email_addresses, 'sanitize_email' );
		}
		else {
			$email['recipient'] = sanitize_email( $email_address );
		}

		// Get Subject
		$email['subject'] = get_the_title( $post_id );
		$email['subject'] = apply_filters( 'picu_collection_mail_subject', $email['subject'] );

		// Get collection URL
		$url = get_draft_permalink( $post_id );

		// Get reply-to information
		$author_email = get_the_author_meta( 'user_email', $post->post_author );
		$email['reply'] = get_the_author_meta( 'display_name', $post->post_author ) . ' <' . $author_email . '>';
		$email['cc'] = $author_email;

		// Compose mail message
		$mail_content['title'] = $email['subject'];
		$mail_content['url'] = $url;
		if ( is_array( $email['recipient'] ) ) {
			$mail_content['collection_hashes'] = get_post_meta( $post_id, '_picu_collection_hashes', true );
		}
		$mail_content['user_email'] = $author_email;
		$mail_content['button_text'] = __( 'View Images', 'picu' );
		$mail_content['footer_notice'] = __( 'Not sure why you received this Email?<br>Please <a href="mailto:{user_email}?Subject=Re: {title}">let us know</a>.', 'picu' );

		// Get the description the photographer has entered
		if ( ! empty( $_POST['picu_delivery_description'] ) ) {
			$description = implode( "\n", array_map( 'sanitize_text_field', explode( "\n", stripslashes( $_POST['picu_delivery_description'] ) ) ) );

			if ( empty( $description ) ) { // Define default Text
				$mail_content['message_text'] = __( "Dear Client,\n\nyour images are ready!\n\nDownload them by following the link below.", 'picu' );
			} else { // Use entered description if there is one
				$mail_content['message_text'] = $description;
			}
		}
		else {
			$description = implode( "\n", array_map( 'sanitize_text_field', explode( "\n", stripslashes( $_POST['picu_collection_description'] ) ) ) );

			if ( empty( $description ) ) { // Define default Text
				$mail_content['message_text'] = sprintf( __( "Dear Client,\n\nFind the images of our photo-shoot \"%s\" here:\n%s\n\nPlease select the ones you like and send your selection back to us.\nWe will start post-production as soon as we have your approval.\n\nSincerely,\n%s", 'picu' ), $email['title'], $url, $user_name  );
			} else { // Use entered description if there is one
				$mail_content['message_text'] = $description;
			}
		}

		// Combine the mail content with the email template
		$email['content'] = picu_get_email_template( $mail_content );
		// This will be an array if more than one email-address

		// Send mail and check if sending is successful
		if ( picu_send_email( $email ) == true ) {

			// Set the post status to "delivered" or "sent"
			if ( ! empty( $_POST['picu_delivery_email_address'] ) ) {
				picu_update_post_status( $post_id, 'delivered' );
				picu_update_collection_history( $post_id, 'delivered', $email['recipient'] );
			}
			else {
				picu_update_post_status( $post_id, 'sent' );
				picu_update_collection_history( $post_id, 'sent', $email['recipient'] );
			}

			// Add success notification
			picu_add_notification( 'picu_mail_sent', 'notice notice-success is-dismissible', __( 'The collection was sent to the client.', 'picu' ) );

		} else {
			// If an error occured, save that in the collection history
			picu_update_collection_history( $post_id, 'error', 'email sending error' );

			// Add error notification
			picu_add_notification( 'picu_mail_error', 'notice notice-error is-dismissible', __( 'There was an error sending the email.', 'picu' ) );
		}

	}
	else {
		if ( $post->post_status == 'delivery-draft' ) {
			// Set the post status to "sent"
			picu_update_post_status( $post_id, 'delivered' );
			// Update collection history
			picu_update_collection_history( $post_id, 'delivered', 'manually' );

			// Add success notification
			picu_add_notification( 'picu_mail_sent', 'notice notice-success is-dismissible', __( 'Your delivery is ready! Make sure to send the link to your client:', 'picu' ) . ' <input type="text" value="' . get_draft_permalink( $post_id ) . '" />' );
		}
		else {
			// Delete emails and hashes, if they have been saved before
			delete_post_meta( $post_id, '_picu_collection_email_address' );
			// TODO: Important for Phase 2: probably not delete those, if the collection has been reopened
			delete_post_meta( $post_id, '_picu_collection_hashes' );
			// Set the post status to "sent"
			picu_update_post_status( $post_id, 'sent' );
			// Update collection history
			picu_update_collection_history( $post_id, 'sent', 'manually' );

			// Add success notification
			picu_add_notification( 'picu_mail_sent', 'notice notice-success is-dismissible', __( 'The collection is ready! Make sure to send the link to your client:', 'picu' ) . ' <input type="text" value="' . get_draft_permalink( $post_id ) . '" />' );
		}
	}
}

add_action( 'save_post_picu_collection', 'picu_collection_send_mail', 10, 2 );


/**
 * Send email to photographer when collection has been approved
 *
 * @since 0.7.1
 */
function picu_send_approval_mail( $post_id, $approval_message = '' ) {

	// Get post object
	$post = get_post( $post_id );

	// Send mail to the collection author
	$recipient = get_the_author_meta( 'user_email', $post->post_author );

	// Override recipient by picu settings, possibly
	$options = get_option( 'picu_settings' );
	if ( ! empty( $options['notification_email'] ) ) {
		$recipient = sanitize_email( $options['notification_email'] );
	}

	// Possibility to change the recipient email
	$email['recipient'] = apply_filters( 'picu_approval_mail_recipient', $recipient, $post_id );

	// Subject is...
	$email['subject'] = sprintf( __( 'Collection "%s" approved' , 'picu' ), sanitize_text_field( $post->post_title ) );
	$email['subject'] = apply_filters( 'picu_approval_mail_subject', $email['subject'] );

	// Get reply-to information
	$client_email = get_post_meta( $post_id, '_picu_collection_email_address', true );
	if ( is_email ( $client_email ) ) {
		$email['reply'] = $client_email;
	}

	// Compose message
	$url = admin_url( 'post.php?post=' . $post_id . '&action=edit' );

	$message = sprintf( __( "Your collection \"%s\" has been approved.\n\n", 'picu' ), sanitize_text_field( $post->post_title ) );

	// Add approval message, if there is one
	// TODO: Add approval fields to the email?
	if ( ! empty( $approval_message ) AND ! picu_is_multi_collection( $post_id ) ) {
		$message .= sprintf( __( "Your client has added the following message:\n%s\n\n", 'picu' ), $approval_message );
	}

	$mail_content['title'] = $email['subject'];
	$mail_content['url'] = $url;
	$mail_content['button_text'] = __( 'View Selection', 'picu' );
	$mail_content['message_text'] = $message;

	$email['content'] = picu_get_email_template( $mail_content );

	if ( picu_send_email( $email ) === true ) {
		// Update collection history
		picu_update_collection_history( $post_id, 'approved' );
	}
	else {
		// Update collection history
		picu_update_collection_history( $post_id, 'error', 'approval error' );
	}
}


/**
 * One function to send all picu emails
 *
 * @since 0.9.4
 *
 * @param $email - array with options
 * $email['recipient'] - email address
 * $email['subject'] - string
 * $email['content'] - email body
 * $email['cc'] = email address
 * $email['reply'] - email address
 *
 * Return true, when email was sent
 */
function picu_send_email( $email ) {

	// Check if we want to sent HTML emails
	$options = get_option( 'picu_settings' );

	if ( isset( $options['send_html_mails'] ) AND 'on' != $options['send_html_mails'] ) {
		$email['headers'][] = "Content-Type: text/plain; charset=UTF-8";
	} else {
		$email['headers'][] = "Content-Type: text/html; charset=UTF-8";
	}

	// Set "CC" header
	if ( isset( $email['cc'] ) AND is_email( $email['cc'] ) ) {
		$email['headers'][] = "CC: " . $email['cc'] . "\r\n";
	}
	// Set "From" header
	$blog_url = parse_url( get_bloginfo( 'url' ) );
	$blog_url = $blog_url['host'];

	$from = array();

	$from['name'] = get_bloginfo( 'name' );
	if ( ! empty( $options['from_name'] ) ) {
		$from['name'] = $options['from_name'];
	}

	$from['address'] = "no-reply@" . $blog_url;
	if ( ! empty( $options['from_email'] ) ) {
		$from['address'] = sanitize_email( $options['from_email'] );
	}

	$from = apply_filters( 'picu_email_from', $from );

	// sanitize_email
	$email['headers'][] = "From: " . sanitize_text_field( $from['name'] ) . " <" . sanitize_email( $from['address'] ) . ">\r\n";

	// Set "Reply-To" header
	if ( isset( $email['reply'] ) AND ! empty( $email['reply'] ) ) {
		$email['headers'][] = "Reply-To: " . $email['reply'] . "\r\n";
	}

	// Remove html entities from subject
	$email['subject'] = html_entity_decode( $email['subject'], ENT_QUOTES, 'UTF-8' );

	// Send individual emails
	if ( is_array( $email['recipient'] ) ) {

		// Check if there is different content for each user
		if ( is_array( $email['content'] ) ) {
			$no_error = true;
			foreach( $email['recipient'] as $email_address ) {
				remove_filter( 'wp_mail', 'wp_staticize_emoji_for_email' );
				if ( wp_mail( $email_address, $email['subject'], $email['content'][$email_address], $email['headers'] ) != true ) {
					$no_error = false;
				}
			}
		}
		// Same email content for every recipient
		else {
			$no_error = true;
			foreach( $email['recipient'] as $email_address) {
				remove_filter( 'wp_mail', 'wp_staticize_emoji_for_email' );
				if ( wp_mail( $email_address, $email['subject'], $email['content'], $email['headers'] ) != true ) {
					$no_error = false;
				}
			}
		}

		// TODO: Check if all mails were send, what do we do if one was not send?!
		return $no_error;
	}
	else {
		remove_filter( 'wp_mail', 'wp_staticize_emoji_for_email' );
		if ( wp_mail( $email['recipient'], $email['subject'], $email['content'], $email['headers'] ) == true ) {
			return true;
		}
	}

	return false;
}


/**
 * Update picu collection history
 *
 * @since 0.9.4
 *
 * @param $post_id
 * @param $event, string - sent, reopened, approved
 * @param $data, string or array - additional data
 */

function picu_update_collection_history( $post_id, $event, $data = NULL ) {

	// Load existing history
	$existing_history = get_post_meta( $post_id, '_picu_collection_history' );

	// Create new history array
	$time = time();
	$new_history["$time"] = array(
		'event' => $event,
		'data' => $data
	);

	// Merge arrays
	if ( is_array( $existing_history ) AND 0 < count( $existing_history ) ) {
		$history = $existing_history[0] + $new_history;
	}
	else {
		$history = $new_history;
	}

	// Save updated history
	update_post_meta( $post_id, '_picu_collection_history', $history );
}


/**
 * Get picu collection history event time
 *
 * @since 0.9.4
 *
 * @param $post_id
 * @param $event, string - sent, reopened, approved
 */
function picu_get_collection_history_event_time( $post_id, $event ) {

	$picu_collection_history = get_post_meta( $post_id, '_picu_collection_history', false );

	// Check if history exists and if it contains anything
	if ( is_array( $picu_collection_history ) AND 0 < count( $picu_collection_history ) ) {

		// Get timestamps
		$keys = array_keys( $picu_collection_history[0] );

		// Check at which timestamp our event existing_history
		foreach( $picu_collection_history[0] as $key => $temp ) {

			// Check all events, get the most recent one
			if ( isset( $temp['event'] ) AND $event == $temp['event'] ) {
				$time = $key; // The final time will be the last $event in the history
			}
		}

		// Check if it is a valid timestamp
		if ( isset( $time ) AND is_numeric( $time ) ) {
			return $time;
		}
	}

	return false;
}


/**
 * Get the number of images in a collection
 *
 * @since 1.6.2
 *
 * @param $post_id - The collection id
 * @return int Number of images in a collection
 */
function picu_get_collection_image_num( $post_id ) {

	$collection_images = get_post_meta( $post_id, '_picu_collection_gallery_ids', true );

	$images = explode( ',', $collection_images );

	if ( ! empty( $images[0] ) ) {
		return count( $images );
	}
	else {
		return 0;
	}
}


/**
 * Get the number of selected images of a collection
 *
 * @since 1.3.4
 *
 * @param $post_id - The collection id
 * @return int Number of selected images (at least once for multi client collections)
 */
function picu_get_selection_count( $post_id ) {

	// For multi client collections
	$picu_collection_hashes = get_post_meta( $post_id, '_picu_collection_hashes', true );

	if ( ! empty( $picu_collection_hashes ) ) {

		$image_ids = [];

		foreach( $picu_collection_hashes as $key => $hash ) {
			$selection = get_post_meta( $post_id, '_picu_collection_selection_' . $key, true );
			if ( ! empty( $selection['selection'] ) ) {
				$image_ids = array_merge( $image_ids, $selection['selection'] );
			}
		}

		return count( array_unique( $image_ids ) );
	}
	// For single collections
	else {
		$selection = get_post_meta( $post_id, '_picu_collection_selection', true );
		return ( isset( $selection['selection'] ) AND is_array( $selection['selection'] ) ) ? count( $selection['selection'] ) : 0;
	}
}


/**
 * Get aprooved filenames
 *
 * @param int $post_id The collection id
 * @param string $ident Identification hash for a multi client collection
 * @param bool $all Return selected at least once or selected by all
 *
 * @since 1.5.0
 */
function picu_get_approved_filenames( $post_id, $ident = '', $all = false ) {

	$img_filenames = '';

	// For multi-client collections, get ids for images that are selected at least once
	$picu_collection_hashes = get_post_meta( $post_id, '_picu_collection_hashes', true );

	if ( ! empty( $ident ) AND ( ! is_array( $picu_collection_hashes ) OR ! array_key_exists( $ident, $picu_collection_hashes ) ) ) {
		return '';
	}

	if ( ! empty( $picu_collection_hashes ) ) {

		$selection_image_ids = [];

		// Get ids for a certain identity
		// First, check if the identity exists
		if ( ! empty( $ident ) AND array_key_exists( $ident, $picu_collection_hashes ) ) {
			// Next, check if there is a selection for this identity
			$selection = get_post_meta( $post_id, '_picu_collection_selection_' . $ident, true );
			if ( ! empty( $selection['selection'] ) ) {
				// Fill filenames
				$selection_image_ids = array_merge( $selection_image_ids, $selection['selection'] );
			}
		}
		// Get ids for all identities
		else {
			// Get selected by all
			if ( $all == true ) {
				$start = true;

				foreach( $picu_collection_hashes as $key => $hash ) {
					$selection = get_post_meta( $post_id, '_picu_collection_selection_' . $key, true );

					if ( ! empty( $selection['selection'] ) ) {
						// Fill filenames
						if ( $start == true ) {
							$selection_image_ids = $selection['selection'];
							$start = false;
						}
						else {
							$selection_image_ids = array_intersect( $selection_image_ids, $selection['selection'] );
						}
					}
					
				}
			}
			// Get selected at least once
			else {
				// Iterate through hashes and get selections
				foreach( $picu_collection_hashes as $key => $hash ) {
					// Fill individual selections
					$selection = get_post_meta( $post_id, '_picu_collection_selection_' . $key, true );
					if ( ! empty( $selection['selection'] ) ) {
						// Fill filenames
						$selection_image_ids = array_merge( $selection_image_ids, $selection['selection'] );
					}
				}
			}
		}

		$selection_image_ids = array_unique( $selection_image_ids );
	}
	// Get selection image ids from single client collection
	else {
		$selection_data = get_post_meta( $post_id, '_picu_collection_selection', true );
		if ( ! empty( $selection_data ) ) {
			$selection_image_ids = $selection_data['selection'];
		}
	}

	// Get filenames
	$filename_separator = ( defined( 'PICU_FILENAME_SEPARATOR' ) ) ? PICU_FILENAME_SEPARATOR : ' ';
	$filename_separator = apply_filters( 'picu_filename_separator', $filename_separator );

	if ( ! empty( $selection_image_ids ) ) {
		// Loop through our IDs to get the filenames
		foreach ( $selection_image_ids as $selection_image_id ) {
			// Load file path to original file
			$attachment = wp_get_attachment_image_src( $selection_image_id, 'full' );

			// Use filename without suffix
			$img_filename = pathinfo( $attachment[0], PATHINFO_FILENAME );

			// Apply filters to the filename
			$img_filename = apply_filters( 'picu_approved_filename', $img_filename, $selection_image_id );

			// Add filename to our string, spearated by our separator
			$img_filenames .= $img_filename . $filename_separator;
		}

		$img_filenames = trim( $img_filenames );
	}

	return $img_filenames;
}


/**
 * Create proof txt file
 * 
 * @since 1.5.0
 */
function picu_create_proof_file( $post_id ) {

	// Exit if you are not allowed to download the file
	if ( ! current_user_can( picu_capability(), $post_id ) ) {
		wp_die( '🚫 ' . __( 'You are not allowed to do that.', 'picu' ) );
	}

	// Get title
	$title = get_the_title( $post_id );

	// Get client email address
	$email = get_post_meta( $post_id, '_picu_collection_email_address', true );

	// Get approval date
	$temp_approval_date = picu_get_collection_history_event_time( $post_id, 'approved' );
	if ( ! empty( $temp_approval_date ) ) {
		$approval_time = wp_date( get_option( 'date_format' ) . ', ' . get_option( 'time_format' ), $temp_approval_date );
	}

	// Get filenames
	$img_filenames = picu_get_approved_filenames( $post_id );

	// Get hashes
	$picu_collection_hashes = get_post_meta( $post_id, '_picu_collection_hashes', true );

	/**
	 * Create and return file for multi client collections
	 */ 
	if ( ! empty( $picu_collection_hashes ) ) {

		$client_selections = [];

		// Iterate through multi clients
		foreach( $picu_collection_hashes as $key => $hash ) {

			$selection = get_post_meta( $post_id, '_picu_collection_selection_' . $key, true );

			$approval_fields = [];

			// Legacy: Add the old approval message to the approval fields array
			if ( ! empty( $selection['approval_message'] ) ) {
				$approval_fields['picu_approval_message'] = [
					'label' => __( 'The following comment was added on approval', 'picu' ),
					'value' => $selection['approval_message'],
				];
			}

			if ( ! empty( $selection['approval_fields'] ) ) {
				$approval_fields = $selection['approval_fields'];
			}

			$client_selections[] = [
				'email' => $hash['email'],
				'time' => wp_date( get_option( 'date_format' ) . ', ' . get_option( 'time_format' ), $hash['time'] ),
				'approval_fields' => $approval_fields,
				'filenames' => picu_get_approved_filenames( $post_id, $key ),
			];

		}

		$proof_file_content = sprintf( __( 'The collection "%s" has been approved', 'picu' ), $title );

		if ( ! empty( $email ) ) {
		/* translators: %s: email address */
		$proof_file_content .= ' ' . sprintf( __( 'by %s','picu' ), $email );
		}

		/* translators: %s: date and time */
		if ( ! empty( $approval_time ) ) {
		$proof_file_content .= ' ' . sprintf( __( 'on %s', 'picu' ), $approval_time );
		}

	$proof_file_content .= '.

* * *

';

$proof_file_content .= __( 'Approved at least once', 'picu' ) .':

' . $img_filenames . '

* * *

Approved by all:

' .  picu_get_approved_filenames( $post_id, $ident = '', true ) . '
';

		if ( ! empty( $client_selections ) ) {
			foreach( $client_selections as $client ) {
$proof_file_content .= '
* * *

' . $client['email']. ' approved the collection on ' . $client['time'] . '.

Approved images:

' . $client['filenames'] . '

';

				// Add custom approval fields
				if ( ! empty( $client['approval_fields'] ) ) {
$proof_file_content .= picu_custom_fields_in_proof_file( $client['approval_fields'] );
				}

			}
		}
	}

	/**
	 * Create and return file for single client collections
	 */ 
	else {

		// Get client comment
		$picu_collection_selection = get_post_meta( $post_id, '_picu_collection_selection', true );

		// Create and return file
		$proof_file_content = sprintf( __( 'The collection "%s" has been approved', 'picu' ), $title );

		if ( ! empty( $email ) ) {
		/* translators: %s: email address */
$proof_file_content .= ' ' . sprintf( __( 'by %s','picu' ), $email );
		}

		/* translators: %s: date and time */
		if ( ! empty( $approval_time ) ) {
$proof_file_content .= ' ' . sprintf( __( 'on %s', 'picu' ), $approval_time );
		}

$proof_file_content .= '.

* * *

';

		// Legacy: Add the old approval message to the approval fields array
		if ( ! empty( $picu_collection_selection['approval_message'] ) ) {
			$picu_collection_selection['approval_fields']['picu_approval_message'] = [
				'label' => __( 'The following comment was added on approval', 'picu' ),
				'value' => $picu_collection_selection['approval_message'],
			];
		}

		// Add custom approval fields
		if ( ! empty( $picu_collection_selection['approval_fields'] ) ) {
$proof_file_content .= picu_custom_fields_in_proof_file( $picu_collection_selection['approval_fields'] );
$proof_file_content .= '
* * *

';
		}

$proof_file_content .= __( 'Approved images', 'picu' ) .':

' . $img_filenames;

	}

	// Filter file content
	$proof_file_content = apply_filters( 'picu_proof_file_content', $proof_file_content, $post_id );

	// Filter file name
	$proof_file_name = apply_filters( 'picu_proof_file_name', __( 'approved-collection', 'picu' ) . '-' . sanitize_title( $title ) . '.txt', $post_id );

	// Send text file
	header( 'Content-Type: application/download' );
	header( 'Content-Disposition: attachment; filename="' . $proof_file_name . '"' );
	echo $proof_file_content;
	exit;
}


/**
 * Add approval fields into the proof file
 * 
 * @since 1.6.5
 * 
 * @param array $approval_fields The approval fields
 */
function picu_custom_fields_in_proof_file( $approval_fields ) {

	$text = '';
	$num = count( $approval_fields );
	$i = 1;
	foreach( $approval_fields as $field ) {
		$text .= $field['label'] . ":\n" . $field['value'] . "\n";
		if ( $num != $i ) {
			$text .= "\n";
		}
		$i++;
	}

	return $text;
}